"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const http_1 = require("http");
const https_1 = require("https");
const mime_db_1 = tslib_1.__importDefault(require("mime-db"));
const url_1 = require("url");
const async_enumerable_1 = require("@reactive-js/async-enumerable");
const disposable_1 = require("@reactive-js/disposable");
const http_2 = require("@reactive-js/http");
const node_1 = require("@reactive-js/node");
const observable_1 = require("@reactive-js/observable");
const option_1 = require("@reactive-js/option");
const pipe_1 = require("@reactive-js/pipe");
const httpContentEncoding_1 = require("./httpContentEncoding");
const httpResponse_1 = require("./httpResponse");
const spyScanner = ([uploaded, total], ev) => ev.type === 1
    ? [ev.data.length, total + uploaded]
    : [-1, total + uploaded];
const send = (request, requestOptions) => {
    const { contentEncoding } = requestOptions, nodeOptions = tslib_1.__rest(requestOptions, ["contentEncoding"]);
    const { content } = request;
    const newContent = option_1.isSome(content) && option_1.isSome(contentEncoding)
        ? Object.assign(Object.assign({}, content), { contentEncodings: [...content.contentEncodings, contentEncoding] }) : content;
    const requestWithAcceptEncodingsAndContentEncoding = Object.assign(Object.assign({}, request), { acceptedEncodings: httpContentEncoding_1.supportedEncodings, content: newContent });
    const { method, uri } = requestWithAcceptEncodingsAndContentEncoding;
    const headers = http_2.httpRequestToUntypedHeaders(requestWithAcceptEncodingsAndContentEncoding);
    const nodeRequestUrl = uri instanceof url_1.URL ? uri : new url_1.URL(uri.toString());
    const nodeRequestOptions = Object.assign(Object.assign({}, nodeOptions), { headers,
        method });
    return uri.protocol === "https:"
        ? https_1.request(nodeRequestUrl, nodeRequestOptions)
        : uri.protocol === "http:"
            ? http_1.request(nodeRequestUrl, nodeRequestOptions)
            : (() => {
                throw new Error();
            })();
};
const createOnSubscribe = (request, requestOptions, encodeContent, decodeHttpContentResponse) => (subscriber) => {
    var _a, _b;
    subscriber.dispatch({ type: 1, request });
    const req = send(request, requestOptions);
    const onError = (cause) => {
        subscriber.dispose({ cause });
    };
    req.on("error", onError);
    const onResponse = (msg) => {
        var _a;
        subscriber.add(() => msg.destroy());
        const response = pipe_1.pipe(http_2.parseHttpResponseFromHeaders((_a = msg.statusCode) !== null && _a !== void 0 ? _a : -1, msg.headers, node_1.createBufferStreamFromReadable(() => msg)), decodeHttpContentResponse);
        const disposableResponse = disposable_1.createDisposableValue(response, _ => {
            msg.destroy();
        });
        subscriber.dispatch({
            type: 4,
            request,
            response: disposableResponse,
        });
    };
    req.on("response", onResponse);
    const reqBody = node_1.createBufferStreamSinkFromWritable(() => req, false);
    const doOnNotify = ([count, total]) => {
        const ev = count < 0
            ? { type: 3, request }
            : { type: 2, request, total };
        subscriber.dispatch(ev);
    };
    const spyEnumerator = pipe_1.pipe(async_enumerable_1.identity(), async_enumerable_1.lift(pipe_1.compose(observable_1.scan(spyScanner, () => [0, 0]), observable_1.onNotify(doOnNotify)))).enumerateAsync(subscriber);
    const contentBody = pipe_1.pipe((_b = (_a = request.content) === null || _a === void 0 ? void 0 : _a.body) !== null && _b !== void 0 ? _b : async_enumerable_1.emptyStream(), async_enumerable_1.lift(observable_1.onNotify(ev => spyEnumerator.dispatch(ev))), encodeContent);
    subscriber.add(spyEnumerator).add(_ => {
        req.abort();
        req.removeAllListeners();
        req.destroy();
    });
    const onContinue = () => {
        subscriber.add(pipe_1.pipe(async_enumerable_1.sink(contentBody, reqBody), observable_1.subscribe(subscriber)));
    };
    if (request.expectContinue) {
        req.on("continue", onContinue);
    }
    else {
        onContinue();
    }
};
const identity = (x) => x;
const requestIsCompressible = (request) => {
    const { content } = request;
    return option_1.isSome(content)
        ? http_2.mediaTypeIsCompressible(mime_db_1.default)(content.contentType)
        : false;
};
class HttpClientImpl extends disposable_1.AbstractDisposable {
    constructor(clientOptions) {
        super();
        this.clientOptions = clientOptions;
    }
    send(request, options = {}) {
        const _a = this.clientOptions, { agent, insecureHTTPParser, maxHeaderSize, shouldEncode: shouldEncodeOption } = _a, zlibOptions = tslib_1.__rest(_a, ["agent", "insecureHTTPParser", "maxHeaderSize", "shouldEncode"]);
        const { acceptedEncodings = httpContentEncoding_1.supportedEncodings } = options;
        const contentEncoding = httpContentEncoding_1.getFirstSupportedEncoding(acceptedEncodings);
        const shouldEncodeOptionResult = option_1.isSome(shouldEncodeOption)
            ? shouldEncodeOption(request)
            : option_1.none;
        const shouldEncode = option_1.isSome(shouldEncodeOptionResult)
            ? shouldEncodeOptionResult
            : requestIsCompressible(request);
        const contentEncoder = option_1.isSome(contentEncoding) && shouldEncode
            ? node_1.transform(httpContentEncoding_1.createEncodingCompressTransform(contentEncoding, zlibOptions))
            : identity;
        const decodeHttpContentResponseWithOption = httpResponse_1.decodeHttpContentResponse(zlibOptions);
        const requestOptions = {
            agent,
            contentEncoding,
            insecureHTTPParser,
            maxHeaderSize,
        };
        return observable_1.createObservable(createOnSubscribe(request, requestOptions, contentEncoder, decodeHttpContentResponseWithOption));
    }
}
exports.creatHttpClient = (clientOptions = {}) => new HttpClientImpl(clientOptions);
const redirectCodes = [
    301,
    302,
    303,
    307,
    308,
];
exports.createDefaultHttpResponseHandler = (httpClient, maxRedirects = 10) => {
    const handleResponse = (status) => {
        var _a;
        if (status.type === 4) {
            const { request, response } = status;
            const { location, preferences, statusCode } = response.value;
            const acceptedEncodings = (_a = preferences === null || preferences === void 0 ? void 0 : preferences.acceptedEncodings) !== null && _a !== void 0 ? _a : [];
            const shouldRedirect = redirectCodes.includes(statusCode) &&
                option_1.isSome(location) &&
                maxRedirects > 0;
            const [newRequest, newAcceptedEncodings] = shouldRedirect
                ? [
                    http_2.createRedirectHttpRequest(response.value)(request),
                ]
                : statusCode === 417
                    ? [Object.assign(Object.assign({}, request), { expectContinue: false })]
                    : statusCode === 415 &&
                        acceptedEncodings.length > 0
                        ? [request, acceptedEncodings]
                        : [request];
            if (request !== newRequest || option_1.isSome(newAcceptedEncodings)) {
                response.dispose();
                return pipe_1.pipe(httpClient.send(newRequest, {
                    acceptedEncodings: newAcceptedEncodings,
                }), exports.createDefaultHttpResponseHandler(httpClient, maxRedirects - 1));
            }
        }
        return observable_1.ofValue(status);
    };
    return observable_1.concatMap(handleResponse);
};
//# sourceMappingURL=httpClient.js.map