"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async_enumerable_1 = require("@reactive-js/async-enumerable");
const http_1 = require("@reactive-js/http");
const node_1 = require("@reactive-js/node");
const observable_1 = require("@reactive-js/observable");
const pipe_1 = require("@reactive-js/pipe");
const writeResponseMessage = (resp) => (response) => {
    resp.statusCode = response.statusCode;
    http_1.writeHttpResponseHeaders(response, (header, value) => resp.setHeader(header, value));
};
const writeResponseContentBody = (resp) => ({ content, }) => {
    var _a;
    const body = (_a = content === null || content === void 0 ? void 0 : content.body) !== null && _a !== void 0 ? _a : async_enumerable_1.emptyStream();
    const responseBodySink = node_1.createBufferStreamSinkFromWritable(() => resp);
    return async_enumerable_1.sink(body, responseBodySink);
};
const defaultOnError = (_) => observable_1.empty();
exports.createHttpRequestListener = (handler, scheduler, options = {}) => {
    const { onError = defaultOnError } = options;
    const handleRequest = (req, resp) => {
        var _a;
        const { method, url: path = "/", headers, httpVersionMajor, httpVersionMinor, } = req;
        const body = node_1.createBufferStreamFromReadable(() => req);
        const isTransportSecure = (_a = req.socket.encrypted) !== null && _a !== void 0 ? _a : false;
        return pipe_1.pipe(() => http_1.parseHttpRequestFromHeaders({
            method: method,
            path,
            headers: headers,
            httpVersionMajor,
            httpVersionMinor,
            isTransportSecure,
            body,
        }), observable_1.compute, observable_1.await_(handler), observable_1.onNotify(writeResponseMessage(resp)), observable_1.await_(writeResponseContentBody(resp)), observable_1.catchError(onError));
    };
    return (req, resp) => {
        const subscription = pipe_1.pipe(handleRequest(req, resp), observable_1.subscribe(scheduler)).add(() => {
            req.removeAllListeners();
            resp.removeAllListeners();
            req.destroy();
        });
        const dispose = () => subscription.dispose();
        req.on("aborted", dispose);
        req.on("close", dispose);
        req.on("error", dispose);
        resp.on("close", dispose);
        resp.on("error", dispose);
        resp.on("finish", dispose);
    };
};
//# sourceMappingURL=httpRequestListener.js.map