"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const mime_db_1 = tslib_1.__importDefault(require("mime-db"));
const http_1 = require("@reactive-js/http");
const option_1 = require("@reactive-js/option");
const httpContent_1 = require("./httpContent");
const httpContentEncoding_1 = require("./httpContentEncoding");
const responseIsCompressible = (response) => {
    const { content } = response;
    return option_1.isSome(content)
        ? http_1.mediaTypeIsCompressible(mime_db_1.default)(content.contentType)
        : false;
};
exports.encodeHttpResponse = (request, options = {}) => response => {
    const { shouldEncode: shouldEncodeOption } = options, zlibOptions = tslib_1.__rest(options, ["shouldEncode"]);
    const shouldEncodeOptionResult = option_1.isSome(shouldEncodeOption)
        ? shouldEncodeOption(request, response)
        : option_1.none;
    const shouldEncode = option_1.isSome(shouldEncodeOptionResult)
        ? shouldEncodeOptionResult
        : responseIsCompressible(response);
    const { preferences } = request;
    const acceptedEncodings = option_1.isSome(preferences) && shouldEncode ? preferences.acceptedEncodings : [];
    const { content, vary } = response;
    const encoding = httpContentEncoding_1.getFirstSupportedEncoding(acceptedEncodings !== null && acceptedEncodings !== void 0 ? acceptedEncodings : []);
    const encodeBody = option_1.isSome(encoding) && option_1.isSome(content);
    return Object.assign(Object.assign({}, response), { content: option_1.isSome(encoding) && option_1.isSome(content)
            ? httpContent_1.encodeHttpContent(content, encoding, zlibOptions)
            : content, vary: encodeBody ? [...vary, "Accept-Encoding"] : vary });
};
exports.decodeHttpContentResponse = (options) => response => {
    const { content } = response;
    return Object.assign(Object.assign({}, response), { content: option_1.isSome(content) ? httpContent_1.decodeHttpContent(content, options) : option_1.none });
};
//# sourceMappingURL=httpResponse.js.map