import iconv from "iconv-lite";
import { ofValueStream, } from "@reactive-js/async-enumerable";
import { createHttpContent, parseMediaTypeOrThrow, } from "@reactive-js/http";
import { transform, createBufferStreamFromReadable } from "@reactive-js/node";
import { pipe } from "@reactive-js/pipe";
import { createEncodingCompressTransform, createEncodingDecompressTransform, } from "./httpContentEncoding";
export const encodeHttpContent = (content, encoding, options) => {
    const { body, contentLength, contentEncodings } = content;
    return contentLength === 0
        ? content
        : Object.assign(Object.assign({}, content), { body: pipe(body, transform(createEncodingCompressTransform(encoding, options))), contentEncodings: [...contentEncodings, encoding], contentLength: -1 });
};
export const decodeHttpContent = (contentBody, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentEncodings.length === 0 || contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: contentEncodings
                .map(encoding => createEncodingDecompressTransform(encoding, options))
                .reduceRight((acc, decoder) => pipe(acc, transform(decoder)), body), contentEncodings: [] });
};
export const createBufferHttpContent = (chunk, contentType) => createHttpContent({
    body: ofValueStream(chunk),
    contentLength: chunk.length,
    contentType,
});
export const createReadableHttpContent = (factory, contentType, contentLength = -1) => createHttpContent({
    body: createBufferStreamFromReadable(factory),
    contentLength,
    contentType,
});
export const createStringHttpContent = (content, contentType) => {
    var _a;
    contentType =
        typeof contentType === "string"
            ? parseMediaTypeOrThrow(contentType)
            : contentType;
    const charset = (_a = contentType.params["charset"]) !== null && _a !== void 0 ? _a : "utf-8";
    const buffer = iconv.encode(content, charset);
    contentType = Object.assign(Object.assign({}, contentType), { params: Object.assign(Object.assign({}, contentType.params), { charset }) });
    return createBufferHttpContent(buffer, contentType);
};
//# sourceMappingURL=httpContent.js.map