import { emptyStream, sink, } from "@reactive-js/async-enumerable";
import { writeHttpResponseHeaders, parseHttpRequestFromHeaders, } from "@reactive-js/http";
import { createBufferStreamFromReadable, createBufferStreamSinkFromWritable, } from "@reactive-js/node";
import { await_, catchError, onNotify, subscribe, empty, compute, } from "@reactive-js/observable";
import { pipe } from "@reactive-js/pipe";
const writeResponseMessage = (resp) => (response) => {
    resp.statusCode = response.statusCode;
    writeHttpResponseHeaders(response, (header, value) => resp.setHeader(header, value));
};
const writeResponseContentBody = (resp) => ({ content, }) => {
    var _a;
    const body = (_a = content === null || content === void 0 ? void 0 : content.body) !== null && _a !== void 0 ? _a : emptyStream();
    const responseBodySink = createBufferStreamSinkFromWritable(() => resp);
    return sink(body, responseBodySink);
};
const defaultOnError = (_) => empty();
export const createHttpRequestListener = (handler, scheduler, options = {}) => {
    const { onError = defaultOnError } = options;
    const handleRequest = (req, resp) => {
        var _a;
        const { method, url: path = "/", headers, httpVersionMajor, httpVersionMinor, } = req;
        const body = createBufferStreamFromReadable(() => req);
        const isTransportSecure = (_a = req.socket.encrypted) !== null && _a !== void 0 ? _a : false;
        return pipe(() => parseHttpRequestFromHeaders({
            method: method,
            path,
            headers: headers,
            httpVersionMajor,
            httpVersionMinor,
            isTransportSecure,
            body,
        }), compute, await_(handler), onNotify(writeResponseMessage(resp)), await_(writeResponseContentBody(resp)), catchError(onError));
    };
    return (req, resp) => {
        const subscription = pipe(handleRequest(req, resp), subscribe(scheduler)).add(() => {
            req.removeAllListeners();
            resp.removeAllListeners();
            req.destroy();
        });
        const dispose = () => subscription.dispose();
        req.on("aborted", dispose);
        req.on("close", dispose);
        req.on("error", dispose);
        resp.on("close", dispose);
        resp.on("error", dispose);
        resp.on("finish", dispose);
    };
};
//# sourceMappingURL=httpRequestListener.js.map