import { __rest } from "tslib";
import db from "mime-db";
import { mediaTypeIsCompressible, } from "@reactive-js/http";
import { isSome, none } from "@reactive-js/option";
import { encodeHttpContent, decodeHttpContent } from "./httpContent";
import { getFirstSupportedEncoding } from "./httpContentEncoding";
const responseIsCompressible = (response) => {
    const { content } = response;
    return isSome(content)
        ? mediaTypeIsCompressible(db)(content.contentType)
        : false;
};
export const encodeHttpResponse = (request, options = {}) => response => {
    const { shouldEncode: shouldEncodeOption } = options, zlibOptions = __rest(options, ["shouldEncode"]);
    const shouldEncodeOptionResult = isSome(shouldEncodeOption)
        ? shouldEncodeOption(request, response)
        : none;
    const shouldEncode = isSome(shouldEncodeOptionResult)
        ? shouldEncodeOptionResult
        : responseIsCompressible(response);
    const { preferences } = request;
    const acceptedEncodings = isSome(preferences) && shouldEncode ? preferences.acceptedEncodings : [];
    const { content, vary } = response;
    const encoding = getFirstSupportedEncoding(acceptedEncodings !== null && acceptedEncodings !== void 0 ? acceptedEncodings : []);
    const encodeBody = isSome(encoding) && isSome(content);
    return Object.assign(Object.assign({}, response), { content: isSome(encoding) && isSome(content)
            ? encodeHttpContent(content, encoding, zlibOptions)
            : content, vary: encodeBody ? [...vary, "Accept-Encoding"] : vary });
};
export const decodeHttpContentResponse = (options) => response => {
    const { content } = response;
    return Object.assign(Object.assign({}, response), { content: isSome(content) ? decodeHttpContent(content, options) : none });
};
//# sourceMappingURL=httpResponse.js.map