/// <reference types="node" />
import { Agent } from "http";
import { ZlibOptions, BrotliOptions } from "zlib";
import { DisposableValueLike, DisposableLike } from "@reactive-js/disposable";
import { HttpContentEncoding, HttpContentRequest, HttpContentResponse } from "@reactive-js/http";
import { BufferStreamLike } from "@reactive-js/node";
import { ObservableLike, ObservableOperator } from "@reactive-js/observable";
import { Option } from "@reactive-js/option";
export declare const enum HttpClientRequestStatusType {
    Begin = 1,
    Uploaded = 2,
    UploadComplete = 3,
    ResponseReady = 4
}
export declare type HttpClientRequestStatusBegin = {
    readonly type: HttpClientRequestStatusType.Begin;
    readonly request: HttpContentRequest<BufferStreamLike>;
};
export declare type HttpClientRequestStatusUploading = {
    readonly type: HttpClientRequestStatusType.Uploaded;
    readonly request: HttpContentRequest<BufferStreamLike>;
    readonly total: number;
};
export declare type HttpClientRequestStatusUploadComplete = {
    readonly type: HttpClientRequestStatusType.UploadComplete;
    readonly request: HttpContentRequest<BufferStreamLike>;
};
export declare type HttpClientRequestStatusResponseReady = {
    readonly type: HttpClientRequestStatusType.ResponseReady;
    readonly request: HttpContentRequest<BufferStreamLike>;
    readonly response: DisposableValueLike<HttpContentResponse<BufferStreamLike>>;
};
export declare type HttpClientRequestStatus = HttpClientRequestStatusBegin | HttpClientRequestStatusUploading | HttpClientRequestStatusUploadComplete | HttpClientRequestStatusResponseReady;
export declare type HttpClientOptions = BrotliOptions & ZlibOptions & {
    readonly agent?: Agent | boolean;
    readonly insecureHTTPParser?: boolean;
    readonly maxHeaderSize?: number;
    readonly shouldEncode?: (req: HttpContentRequest<unknown>) => Option<boolean>;
};
export declare type HttpClientRequestOptions = {
    readonly acceptedEncodings?: readonly HttpContentEncoding[];
};
export interface HttpClientLike extends DisposableLike {
    send(request: HttpContentRequest<BufferStreamLike>, requestOptions?: HttpClientRequestOptions): ObservableLike<HttpClientRequestStatus>;
}
export declare const creatHttpClient: (clientOptions?: HttpClientOptions) => HttpClientLike;
export declare const createDefaultHttpResponseHandler: (httpClient: HttpClientLike, maxRedirects?: number) => ObservableOperator<HttpClientRequestStatus, HttpClientRequestStatus>;
//# sourceMappingURL=httpClient.d.ts.map